import TinyDatePicker from 'tiny-date-picker';
import DateRangePicker from 'tiny-date-picker/dist/date-range-picker';
import moment from 'moment';
export class CwcDatepicker {
    constructor() {
        this.langConstants = {
            en: {
                days: 'Sun_Mon_Tue_Wed_Thu_Fri_Sat'.split('_'),
                months: 'January_February_March_April_May_June_July_August_September_October_November_December'.split('_'),
                today: 'Today',
                clear: 'Clear',
                close: 'Close',
                startDate: 'Start date',
                endDate: 'End date',
                selectDate: 'Select date',
            },
            de: {
                days: 'So._Mo._Di._Mi._Do._Fr._Sa.'.split('_'),
                months: 'Januar_Februar_März_April_Mai_Juni_Juli_August_September_Oktober_November_Dezember'.split('_'),
                today: 'Heute',
                clear: 'Klar',
                close: 'Schließen',
                startDate: 'Frühestens',
                endDate: 'Spätestens',
                selectDate: 'Datum auswählen',
            },
        };
        this.range = false;
        this.lang = 'en';
        this.format = 'MM.DD.YYYY';
        this.isReadonly = false;
        this.viewMode = false;
        this.initDatePicker = () => {
            this.datePicker = TinyDatePicker(this.pickerInputElement, {
                lang: this.langConstants[this.lang] || this.langConstants['en'],
                mode: 'dp-below',
            }).on('statechange', (_, { state }) => {
                const { selectedDate } = state;
                this.statechange.emit({ selectedDate });
                this.onChange.emit({
                    id: this.id,
                    value: selectedDate ? this.selectDateValueString(selectedDate) : null,
                });
            });
        };
        this.initDateRangePicker = () => {
            this.dateRangePicker = DateRangePicker.DateRangePicker(this.container, {
                startOpts: {
                    start: this.startDate ? new Date(this.startDate) : '',
                    end: this.endDate ? new Date(this.endDate) : '',
                    lang: this.langConstants[this.lang] || this.langConstants['en'],
                },
            }).on('statechange', (_, { state }) => {
                const { start, end } = state;
                this.pickerInputElement.value = this.formInputValueString(start, end);
                this.statechange.emit({ start, end });
                this.onChange.emit({
                    id: this.id,
                    value: {
                        endDate: end ? this.endDateValueString(end) : null,
                        startDate: start ? this.startDateValueString(start) : null,
                    },
                });
            });
            this.pickerInputElement.addEventListener('focus', () => this.showRangePicker());
            this.pickerInputElement.addEventListener('change', () => {
                const parsed = this.parseInputString(this.pickerInputElement.value);
                const dateStart = new Date(parsed[0]);
                const dateEnd = new Date(parsed[1]);
                this.dateRangePicker.setState({
                    start: !isNaN(dateStart.getTime()) ? dateStart : '',
                    end: !isNaN(dateEnd.getTime()) ? dateEnd : '',
                });
            });
            let previousTimeout;
            this.el.addEventListener('focusout', () => {
                clearTimeout(previousTimeout);
                previousTimeout = setTimeout(() => {
                    if (!this.el.contains(document.activeElement)) {
                        this.container.classList.remove('ex-inputs-picker-visible');
                    }
                }, 10);
            });
        };
    }
    clear() {
        this.range
            ? this.dateRangePicker.setState({ start: '', end: '' })
            : this.datePicker.setState({ selectedDate: '' });
    }
    componentDidLoad() {
        this.container = this.el.querySelector('.ex-inputs-picker');
        this.pickerInputElement = this.el.querySelector('.ex-input-datepicker');
        this.range ? this.initDateRangePicker() : this.initDatePicker();
    }
    parseInputString(value) {
        const values = value.split('-');
        return values.map(val => val.trim());
    }
    formInputValueString(startDate, endDate) {
        return `${this.startDateValueString(startDate)} - ${this.endDateValueString(endDate)}`;
    }
    startDateValueString(startDate) {
        return startDate
            ? moment(startDate).format(this.format)
            : this.langConstants[this.lang].startDate;
    }
    endDateValueString(endDate) {
        return endDate ? moment(endDate).format(this.format) : this.langConstants[this.lang].endDate;
    }
    selectDateValueString(selectDate) {
        return selectDate
            ? moment(selectDate).format(this.format)
            : this.langConstants[this.lang].selectDate;
    }
    showRangePicker() {
        this.container.classList.add('ex-inputs-picker-visible');
    }
    render() {
        console.log(this.isReadonly);
        return (h("div", { class: `
          form-group ex-inputs
          ${this.isReadonly ? 'readonly' : ''}
          ${this.viewMode ? 'viewmode' : ''}
        ` },
            this.label && h("label", { class: "control-label" }, this.label),
            h("div", { class: "ex-input-datepicker-wrapper" },
                h("input", { id: this.id, class: "form-control ex-input-datepicker", placeholder: this.range
                        ? `${this.langConstants[this.lang].startDate} - ${this.langConstants[this.lang].endDate}`
                        : `${this.langConstants[this.lang].selectDate}`, readonly: this.isReadonly && this.viewMode }),
                h("button", { id: "datepicker-clearable-btn", class: "datepicker-clear-btn", onClick: () => this.clear() }, "\u00D7")),
            h("div", { class: "ex-inputs-picker" })));
    }
    static get is() { return "cwc-datepicker"; }
    static get properties() { return {
        "clear": {
            "method": true
        },
        "el": {
            "elementRef": true
        },
        "element": {
            "elementRef": true
        },
        "endDate": {
            "type": String,
            "attr": "end-date"
        },
        "format": {
            "type": String,
            "attr": "format"
        },
        "id": {
            "type": String,
            "attr": "id"
        },
        "isReadonly": {
            "type": Boolean,
            "attr": "is-readonly"
        },
        "label": {
            "type": String,
            "attr": "label"
        },
        "lang": {
            "type": String,
            "attr": "lang"
        },
        "range": {
            "type": Boolean,
            "attr": "range"
        },
        "startDate": {
            "type": String,
            "attr": "start-date"
        },
        "viewMode": {
            "type": Boolean,
            "attr": "view-mode"
        }
    }; }
    static get events() { return [{
            "name": "statechange",
            "method": "statechange",
            "bubbles": true,
            "cancelable": true,
            "composed": true
        }, {
            "name": "onChange",
            "method": "onChange",
            "bubbles": true,
            "cancelable": true,
            "composed": true
        }]; }
    static get style() { return "/**style-placeholder:cwc-datepicker:**/"; }
}
